@@ -1,8 +1,11 @@
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
+
 /* Metacity X window decorations */
 
 /* 
  * Copyright (C) 2001 Havoc Pennington
  * Copyright (C) 2003, 2004 Red Hat, Inc.
+ * Copyright (C) 2005 Elijah Newren
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -38,60 +41,6 @@
                     EnterWindowMask | LeaveWindowMask |            \
                     FocusChangeMask |                              \
                     ColormapChangeMask)
-static Visual*
-find_argb_visual (MetaDisplay *display,
-                  int          scr)
-{
-#ifdef HAVE_RENDER
-  XVisualInfo		*xvi;
-  XVisualInfo		template;
-  int			nvi;
-  int			i;
-  XRenderPictFormat	*format;
-  Visual		*visual;
-
-  if (!META_DISPLAY_HAS_RENDER (display))
-    return NULL;
-  
-  template.screen = scr;
-  template.depth = 32;
-  template.class = TrueColor;
-  xvi = XGetVisualInfo (display->xdisplay, 
-                        VisualScreenMask |
-                        VisualDepthMask |
-                        VisualClassMask,
-                        &template,
-                        &nvi);
-  if (!xvi)
-    return 0;
-  
-  visual = NULL;
-
-  for (i = 0; i < nvi; i++)
-    {
-      format = XRenderFindVisualFormat (display->xdisplay, xvi[i].visual);
-      if (format->type == PictTypeDirect && format->direct.alphaMask)
-	{
-          visual = xvi[i].visual;
-          break;
-	}
-    }
-
-  XFree (xvi);
-
-  if (visual)
-    meta_topic (META_DEBUG_COMPOSITOR,
-                "Found ARGB visual 0x%lx\n",
-                (long) visual->visualid);
-  else
-    meta_topic (META_DEBUG_COMPOSITOR,
-                "No ARGB visual found\n");
-  
-  return visual;
-#else  /* RENDER */
-  return NULL;
-#endif /* !RENDER */
-}
 
 void
 meta_window_ensure_frame (MetaWindow *window)
@@ -102,7 +51,7 @@ meta_window_ensure_frame (MetaWindow *window)
   
   if (window->frame)
     return;
-
+  
   /* See comment below for why this is required. */
   meta_display_grab (window->display);
   
@@ -142,14 +91,13 @@ meta_window_ensure_frame (MetaWindow *window)
    */
   
   /* Special case for depth 32 windows (assumed to be ARGB),
-   * we use the window's visual
+   * we use the window's visual. Otherwise we just use the system visual.
    */
   if (window->depth == 32)
     visual = window->xvisual;
   else
-    visual = find_argb_visual(window->display, 
-                              window->screen->number);
-
+    visual = NULL;
+  
   frame->xwindow = meta_ui_create_frame_window (window->screen->ui,
                                                 window->display->xdisplay,
                                                 visual,
@@ -166,6 +114,11 @@ meta_window_ensure_frame (MetaWindow *window)
   
   meta_display_register_x_window (window->display, &frame->xwindow, window);
 
+  /* Now that frame->xwindow is registered with window, we can set its
+   * background.
+   */
+  meta_ui_reset_frame_bg (window->screen->ui, frame->xwindow);
+
   /* Reparent the client window; it may be destroyed,
    * thus the error trap. We'll get a destroy notify later
    * and free everything. Comment in FVWM source code says
@@ -249,7 +202,10 @@ meta_window_destroy_frame (MetaWindow *window)
   XReparentWindow (window->display->xdisplay,
                    window->xwindow,
                    window->screen->xroot,
-                   /* FIXME where to put it back depends on the gravity */
+                   /* Using anything other than meta_window_get_position()
+                    * coordinates here means we'll need to ensure a configure
+                    * notify event is sent; see bug 399552.
+                    */
                    window->frame->rect.x,
                    window->frame->rect.y);
   meta_error_trap_pop (window->display, FALSE);
@@ -267,7 +223,7 @@ meta_window_destroy_frame (MetaWindow *window)
   g_free (frame);
   
   /* Put our state back where it should be */
-  meta_window_queue_calc_showing (window);
+  meta_window_queue (window, META_QUEUE_CALC_SHOWING);
 }
 
 
@@ -319,7 +275,10 @@ meta_frame_get_flags (MetaFrame *frame)
   if (frame->window->on_all_workspaces)
     flags |= META_FRAME_STUCK;
 
-  if (frame->window->maximized)
+  /* FIXME: Should we have some kind of UI for windows that are just vertically
+   * maximized or just horizontally maximized?
+   */
+  if (META_WINDOW_MAXIMIZED (frame->window))
     flags |= META_FRAME_MAXIMIZED;
 
   if (frame->window->fullscreen)
@@ -328,6 +287,9 @@ meta_frame_get_flags (MetaFrame *frame)
   if (frame->is_flashing)
     flags |= META_FRAME_IS_FLASHING;
 
+  if (frame->window->wm_state_above)
+    flags |= META_FRAME_ABOVE;
+  
   return flags;
 }
 
