@@ -109,20 +109,7 @@ static int   cursor_size = 24;
 static gboolean compositing_manager = FALSE;
 
 static MetaVisualBellType visual_bell_type = META_VISUAL_BELL_FULLSCREEN_FLASH;
-static MetaButtonLayout button_layout = {
-  {
-    META_BUTTON_FUNCTION_MENU,
-    META_BUTTON_FUNCTION_LAST,
-    META_BUTTON_FUNCTION_LAST,
-    META_BUTTON_FUNCTION_LAST
-  },
-  {
-    META_BUTTON_FUNCTION_MINIMIZE,
-    META_BUTTON_FUNCTION_MAXIMIZE,
-    META_BUTTON_FUNCTION_CLOSE,
-    META_BUTTON_FUNCTION_LAST
-  }
-};
+static MetaButtonLayout button_layout;
 
 /* The screenshot commands are at the end */
 static char *commands[MAX_COMMANDS] = { NULL, };
@@ -144,7 +131,7 @@ static gboolean update_visual_bell_type   (const char *value);
 static gboolean update_num_workspaces     (int         value);
 static gboolean update_application_based  (gboolean    value);
 static gboolean update_disable_workarounds (gboolean   value);
-static gboolean update_action_double_click_titlebar (const char *value);
+static gboolean update_action_titlebar    (const char *key, const char *value, MetaActionTitlebar *action);
 static gboolean update_auto_raise          (gboolean   value);
 static gboolean update_auto_raise_delay    (int        value);
 static gboolean update_button_layout      (const char *value);
@@ -195,6 +182,7 @@ static gboolean update_list_binding       (MetaKeyPref *binding,
 static void     init_bindings             (void);
 static void     init_commands             (void);
 static void     init_workspace_names      (void);
+static void     init_button_layout        (void);
 
        
 typedef struct
@@ -353,6 +341,20 @@ get_bool (const char *key, gboolean *val)
 
   return filled_in;
 }
+
+static void
+init_action_meta_prefs(const char *key, MetaActionTitlebar *action)
+{
+  GError *err = NULL;
+  char *str_val;
+  
+  str_val = gconf_client_get_string (default_client,
+                                     key,
+                                     &err);
+  cleanup_error (&err);
+  update_action_titlebar (key, str_val, action);
+  g_free (str_val);
+}
 #endif /* HAVE_GCONF */
 
 void
@@ -413,12 +415,9 @@ meta_prefs_init (void)
   if (get_bool (KEY_RAISE_ON_CLICK, &bool_val))
     update_raise_on_click (bool_val);
   
-  str_val = gconf_client_get_string (default_client,
-                                     KEY_ACTION_DOUBLE_CLICK_TITLEBAR,
-				     &err);
-  cleanup_error (&err);
-  update_action_double_click_titlebar (str_val);
-  g_free (str_val);
+  init_action_meta_prefs (KEY_ACTION_DOUBLE_CLICK_TITLEBAR, &action_double_click_titlebar);
+  init_action_meta_prefs (KEY_ACTION_MIDDLE_CLICK_TITLEBAR, &action_middle_click_titlebar);
+  init_action_meta_prefs (KEY_ACTION_RIGHT_CLICK_TITLEBAR, &action_right_click_titlebar);
 
   if (get_bool (KEY_AUTO_RAISE, &bool_val))
     update_auto_raise (bool_val);
@@ -519,6 +518,8 @@ meta_prefs_init (void)
    */
   titlebar_font = pango_font_description_from_string ("Sans Bold 10");
   current_theme = g_strdup ("Atlanta");
+  
+  init_button_layout();
 #endif /* HAVE_GCONF */
   
   /* Load keybindings prefs */
@@ -558,6 +559,23 @@ meta_prefs_init (void)
 
 #ifdef HAVE_GCONF
 
+static gboolean
+action_change_titlebar (const char *key, GConfValue *value, MetaActionTitlebar *action)
+{
+  const char *str;
+
+  if (value && value->type != GCONF_VALUE_STRING)
+    {
+      meta_warning (_("GConf key \"%s\" is set to an invalid type\n"),
+                    key);
+      return FALSE;
+    }
+
+  str = value ? gconf_value_get_string (value) : NULL;
+
+  return update_action_titlebar (key, str, action);
+}
+
 static void
 change_notify (GConfClient    *client,
                guint           cnxn_id,
@@ -806,19 +824,21 @@ change_notify (GConfClient    *client,
     }
   else if (strcmp (key, KEY_ACTION_DOUBLE_CLICK_TITLEBAR) == 0)
     {
-      const char *str;
-
-      if (value && value->type != GCONF_VALUE_STRING)
-        {
-          meta_warning (_("GConf key \"%s\" is set to an invalid type\n"),
-                        key);
-          goto out;
-        }
-
-      str = value ? gconf_value_get_string (value) : NULL;
-
-      if (update_action_double_click_titlebar (str))
+      if (action_change_titlebar(KEY_ACTION_DOUBLE_CLICK_TITLEBAR, value, &action_double_click_titlebar))
         queue_changed (META_PREF_ACTION_DOUBLE_CLICK_TITLEBAR);
+      return;
+    }
+  else if (strcmp (key, KEY_ACTION_MIDDLE_CLICK_TITLEBAR) == 0)
+    {
+      if (action_change_titlebar(KEY_ACTION_MIDDLE_CLICK_TITLEBAR, value, &action_middle_click_titlebar))
+        queue_changed (META_PREF_ACTION_MIDDLE_CLICK_TITLEBAR);
+      return;
+    }
+  else if (strcmp (key, KEY_ACTION_RIGHT_CLICK_TITLEBAR) == 0)
+    {
+      if (action_change_titlebar(KEY_ACTION_RIGHT_CLICK_TITLEBAR, value, &action_right_click_titlebar))
+        queue_changed (META_PREF_ACTION_RIGHT_CLICK_TITLEBAR);
+      return;
     }
   else if (strcmp (key, KEY_AUTO_RAISE) == 0)
     {
@@ -1431,14 +1451,6 @@ update_button_layout (const char *value)
   if (value == NULL)
     return FALSE;
   
-  i = 0;
-  while (i < MAX_BUTTONS_PER_CORNER)
-    {
-      new_layout.left_buttons[i] = META_BUTTON_FUNCTION_LAST;
-      new_layout.right_buttons[i] = META_BUTTON_FUNCTION_LAST;
-      ++i;
-    }
-
   /* We need to ignore unknown button functions, for
    * compat with future versions
    */
@@ -1485,6 +1497,8 @@ update_button_layout (const char *value)
           ++b;
         }
 
+      new_layout.left_buttons[i] = META_BUTTON_FUNCTION_LAST;
+      
       g_strfreev (buttons);
     }
 
@@ -1528,11 +1542,32 @@ update_button_layout (const char *value)
           ++b;
         }
 
+      new_layout.right_buttons[i] = META_BUTTON_FUNCTION_LAST;
+      
       g_strfreev (buttons);
     }
 
   g_strfreev (sides);
   
+  /* Invert the button layout for RTL languages */
+  if (meta_ui_get_direction() == META_UI_DIRECTION_RTL)
+  {
+    MetaButtonLayout rtl_layout;
+    int j;
+    
+    for (i = 0; new_layout.left_buttons[i] != META_BUTTON_FUNCTION_LAST; i++);
+    for (j = 0; j < i; j++)
+	  rtl_layout.right_buttons[j] = new_layout.left_buttons[i - j - 1];
+    rtl_layout.right_buttons[j] = META_BUTTON_FUNCTION_LAST;
+      
+    for (i = 0; new_layout.right_buttons[i] != META_BUTTON_FUNCTION_LAST; i++);
+    for (j = 0; j < i; j++)
+      rtl_layout.left_buttons[j] = new_layout.right_buttons[i - j - 1];
+    rtl_layout.left_buttons[j] = META_BUTTON_FUNCTION_LAST;
+
+    new_layout = rtl_layout;
+  }
+  
   changed = !button_layout_equal (&button_layout, &new_layout);
 
   button_layout = new_layout;
@@ -1653,23 +1688,23 @@ action_titlebar_from_string (const char *str)
 }
 
 static gboolean
-update_action_double_click_titlebar (const char *value)
+update_action_titlebar (const char *key, const char *value, MetaActionTitlebar *action)
 {
-  MetaActionTitlebar old_action = action_double_click_titlebar;
+  MetaActionTitlebar old_action = *action, new_action = old_action;
   
   if (value != NULL)
     {
-      action_double_click_titlebar = action_titlebar_from_string (value);
+      new_action = action_titlebar_from_string (value);
 
-      if (action_double_click_titlebar == META_ACTION_TITLEBAR_LAST)
+      if (new_action == META_ACTION_TITLEBAR_LAST)
         {
-          action_double_click_titlebar = old_action;
-          meta_warning (_("GConf key '%s' is set to an invalid value\n"),
-                        KEY_ACTION_DOUBLE_CLICK_TITLEBAR);
+          new_action = old_action;
+          meta_warning (_("GConf key '%s' is set to an invalid value\n"), key);
         }
+      *action = new_action;
     }
 
-  return (old_action != action_double_click_titlebar);
+  return (old_action != new_action);
 }
 
 static gboolean
@@ -1765,6 +1800,12 @@ meta_preference_to_string (MetaPreference pref)
     case META_PREF_ACTION_DOUBLE_CLICK_TITLEBAR:
       return "ACTION_DOUBLE_CLICK_TITLEBAR";
 
+    case META_PREF_ACTION_MIDDLE_CLICK_TITLEBAR:
+      return "ACTION_MIDDLE_CLICK_TITLEBAR";
+
+    case META_PREF_ACTION_RIGHT_CLICK_TITLEBAR:
+      return "ACTION_RIGHT_CLICK_TITLEBAR";
+
     case META_PREF_AUTO_RAISE:
       return "AUTO_RAISE";
       
@@ -1910,6 +1951,7 @@ static MetaKeyPref screen_bindings[] = {
   { META_KEYBINDING_COMMAND_SCREENSHOT, NULL, FALSE },
   { META_KEYBINDING_COMMAND_WIN_SCREENSHOT, NULL, FALSE },
   { META_KEYBINDING_RUN_COMMAND_TERMINAL, NULL, FALSE },
+  { META_KEYBINDING_SET_SPEW_MARK, NULL, FALSE },
   { NULL, NULL, FALSE}
 };
 
@@ -2999,3 +3041,39 @@ meta_prefs_get_compositing_manager (void)
 {
   return compositing_manager;
 }
+
+static void
+init_button_layout(void)
+{
+  MetaButtonLayout button_layout_ltr = {
+    {    
+      /* buttons in the group on the left side */
+      META_BUTTON_FUNCTION_MENU,
+      META_BUTTON_FUNCTION_LAST
+    },
+    {
+      /* buttons in the group on the right side */
+      META_BUTTON_FUNCTION_MINIMIZE,
+      META_BUTTON_FUNCTION_MAXIMIZE,
+      META_BUTTON_FUNCTION_CLOSE,
+      META_BUTTON_FUNCTION_LAST
+    }
+  };
+  MetaButtonLayout button_layout_rtl = {
+    {    
+      /* buttons in the group on the left side */
+      META_BUTTON_FUNCTION_CLOSE,
+      META_BUTTON_FUNCTION_MAXIMIZE,
+      META_BUTTON_FUNCTION_MINIMIZE,
+      META_BUTTON_FUNCTION_LAST
+    },
+    {
+      /* buttons in the group on the right side */
+      META_BUTTON_FUNCTION_MENU,
+      META_BUTTON_FUNCTION_LAST
+    }
+  };
+
+  button_layout = meta_ui_get_direction() == META_UI_DIRECTION_LTR ?
+    button_layout_ltr : button_layout_rtl;
+};
